<?php
/**
 * @file
 * Definition for CKContentInjectorTextParser
 */

/**
 * Text parsing helper class.
 *
 * Hold regex and matches variables.
 * Finds lines with appropriate text as per regex.
 * Builds replacement string and replaces all the lines found.
 */
class CKContentInjectorTextParser {

  protected static $patternTemplate = '/(?:<p>\s*)?\[\[(TEMPLATE\s*([^\]]*?)\s*)\]\](?:\s*<\/p>)?/';
  protected static $pathTemplate = '';
  protected $text;
  protected $node;
  protected $results;
  protected $callbackData;

  /**
   * Constructor function.
   *
   * @param string $text
   *   The text to parse
   * @param object $node
   *   Current node
   */
  function __construct($text, $node) {
    $this->results = array();
    $this->text = $text;
    $this->node = $node;
    $module_path = drupal_get_path('module', 'ck_content_injector');
    self::$pathTemplate = $module_path . '/injector_templates/';
  }

  /**
   * Process the text for a particular type of token.
   */
  public function process() {
    $matches = array();
    preg_match_all(self::$patternTemplate, $this->text, $matches);
    $this->buildReplacements($matches);
  }

  /**
   * Get the array containing the replacements, suitable for passing
   * straight to Token.
   *
   * @return array
   *   Results array: key = original value, value = replacement
   */
  public function getResults() {
    return $this->results;
  }

  /**
   * Clear the array containing the replacements, for resetting the
   * object (for Unit Tests).
   */
  public function clearResults() {
    $this->results = array();
  }

  /**
   * Build text replacements in string.
   *
   * The regex returns 2 strings for each match, one with the match
   * another with the second part of the matched string, used for
   * storing substitution arguments:
   * $matches[0][0]= [[TEMPLATE injector,param,param,... ]]
   * $matches[1][0]= injector,param,param,...
   *
   * @param array $matches
   *   Matches to replace (from preg_match)
   */
  protected function buildReplacements($matches) {
    $size = sizeof($matches[0]);
    for ($i = 0; $i < $size; $i++) {
      $this->results[$matches[1][$i]] = $this->replacement($matches[2][$i]);
    }
  }

  /**
   * Use the arguments from the string found and construct the replacement
   * string based on $this->mode.
   *
   * @param string $match
   *   Injector tag
   *
   * @return string
   *   Adjusted replacement string
   */
  protected function replacement($match) {
        return $this->replaceTemplate($match);
  }

  /**
   * Manually re-run the replacements for embedded tokens within the injectors.
   *
   * @param object $object
   *   Main object containing content
   *
   * @return string
   *   HTML with replacements
   */
  protected function replace($object) {
    $html = $object->body;
    $text_parser = new CK_content_injector_TextParser($html, $object);
    $text_parser->process();
    $values = $text_parser->get_results();
    foreach ($values as $key => $val) {
      $html = str_replace('[[' . $key . ']]', $val, $html);
    }
    return $html;
  }

  /**
   * Replace a TEMPLATE tag with the appropriate content
   *
   * @param string $match
   *   The contents of the matched TEMPLATE element
   *
   * @return string
   *   The replacement string
   */
  protected function replaceTemplate($match) {
    $parts = explode(',', $match);
    // Parameter 0 is the template name.
    $template_name = $parts[0];
    if (file_exists(self::$pathTemplate . $template_name . '.php')) {
      // Get template params definition array.
      $fields = $this->getTemplateParamsDefinition($template_name);
      // Parameters are made available through the $parameter_values array.
      $parameter_values = array();
      for ($i = 0; $i < count($parts) - 1; $i++) {
        $parameter_values[$i] = rawurldecode($parts[$i + 1]);
        if (!empty($fields[$i]['field_name'])) {
          $parameter_values[$fields[$i]['field_name']] = rawurldecode($parts[$i + 1]);
        }
      }
      ob_start();
      include self::$pathTemplate . $template_name . '.php';
      $replacement = ob_get_contents();
      ob_end_clean();
    }
    else {
      // Template file not found.
      $replacement = '';
    }
    return $replacement;
  }

  /**
   * Get the params array definition for the template.
   *
   * @param string $template_name
   *   Template file name
   *
   * @return array
   *   Template parameter definition
   */
  protected function getTemplateParamsDefinition($template_name) {
    ob_start();
    include self::$pathTemplate . $template_name . '.php';
    ob_end_clean();
    $fields = array();
    // Ensure fields are in order.
    if (!empty($params) && is_array($params)) {
      foreach ($params as $f) {
        $fields[] = $f;
      }
    }
    return $fields;
  }
}
